/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2007-2010 Lucian Langa <cooly@gnome.eu.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gi18n-lib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>

#include <gtk/gtk.h>

#if (DATASERVER_VERSION >= 2031001)
#include <camel/camel.h>
#else
#include <camel/camel-mime-message.h>
#include <camel/camel-folder.h>
#include <camel/camel-exception.h>
#include <camel/camel-multipart.h>
#include <camel/camel-stream-mem.h>
#include <camel/camel-stream-fs.h>
#endif

#if EVOLUTION_VERSION < 22900
#include <mail/mail-component.h>
#include <mail/em-popup.h>
#include <misc/e-activity-handler.h>
#else
#include <mail/e-mail-local.h>
#include <mail/message-list.h>
#include <misc/e-popup-action.h>
#include <shell/e-shell-view.h>
#include <shell/es-event.h>
#endif

#include <e-util/e-mktemp.h>
#include <mail/em-config.h>
#include <mail/mail-config.h>

#include <libedataserver/e-account-list.h>
#include <libedataserver/e-data-server-util.h>

#include <mail/em-utils.h>
#include <mail/mail-ops.h>

#include "rspam.h"

#define d(x)

#if EVOLUTION_VERSION >= 22900 //kb//
extern CamelSession *session;
EShellView *main_shell_view;
#endif

struct rspam_stuff {
	EMConfigTargetPrefs *target;
	GtkWidget *option_menu;
	GtkWidget *option_menu2;
	GtkWidget *option_menu3;
	GtkWidget *option_menu4;
	GtkWidget *username;
	GtkWidget *password;
	GtkWidget *home;
	gchar *homepath;
	gchar *user;
	gchar *pass;
	gchar *spamcop_email;
};

void spamcop_report(CamelMimeMessage *msg, gchar *msgfile);
void taskbar_push_message(gchar *message);
void taskbar_pop_message(void);
void
#if (DATASERVER_VERSION >= 2031002)
rs_online(EShell *shell);
#else
rs_online(CamelObject *o, void *event_data, void *data);
#endif

#define DISCOVERY_TTL		60*60*24*7
#define CNETS			4
#define SPAMCOP_LIMIT		50*1024
//temporary use spamassassin default address
//as spamcop guys did not respond
#define SPAMCOP_DEFAULT_EMAIL	"spamassassin-submit@spam.spamcop.net"

#define RAZORLINE RAZOREP

//colaborative network	spam command line		ham command line	gconf flag name
//		name
char *cnets[CNETS][4] = {
	{"razor",	"razor-report -f -d",		"razor-revoke -f -d",	GCONF_KEY_ENABLE_RAZOR},
	{"dcc",	"dccproc -H -t many -x 0",	NULL,			GCONF_KEY_ENABLE_DCC},
	{"spamcop",(gchar *)spamcop_report,	NULL,			GCONF_KEY_ENABLE_SPAMCOP},
	{"pyzor",  "pyzor -d report",		"pyzor -d whitelist",	GCONF_KEY_ENABLE_PYZOR}};

gchar *pyzor_status = NULL;
GConfClient *rspam_gconf;
struct rspam_stuff *stuff;
gboolean rspam_online;
GList *tlist = NULL;

static void
readrazorconfig(gpointer data);

//spam functions wrapper
//for calling internal functions
void
intern_call(void (*p)(), CamelMimeMessage *msg, gchar *msgfile)
{
	p(msg, msgfile);
}

/* Save email content as a file */
gchar*
save_message(CamelMimeMessage *msg)
{
	CamelDataWrapper *containee;
	CamelStream *out;
	char *tmpdir = NULL, *name = NULL;
	tmpdir = e_mkdtemp("rspam-XXXXXX");
	if ( tmpdir == NULL)
		return (gchar *)-1;

	name = g_build_filename(tmpdir, ".evo-attachment.rspam", NULL);
	out = camel_stream_fs_new_with_name(name, O_RDWR|O_CREAT,
#if EVOLUTION_VERSION < 23191
                                0666);
#else
                                0666, NULL);
#endif

	if (!out)
		goto fail;

	// dump the whole message incl headers to report
#if (DATASERVER_VERSION >= 2033001)
	if (camel_data_wrapper_decode_to_stream_sync(
		(CamelDataWrapper *)msg, out, NULL, NULL) == -1
	   || camel_stream_close(out, NULL, NULL) == -1) {
#else
#if EVOLUTION_VERSION < 23191
	if (camel_data_wrapper_decode_to_stream(
		(CamelDataWrapper *)msg, out) == -1
	   || camel_stream_close(out) == -1) {
#else
	if (camel_data_wrapper_decode_to_stream(
		(CamelDataWrapper *)msg, out, NULL) == -1
	   || camel_stream_close(out, NULL) == -1) {
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(out);
#else
		camel_object_unref(out);
#endif
		goto fail;
	}
#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(out);
#else
	camel_object_unref(out);
#endif
	return name;

fail:
	g_free(name);
	g_free(tmpdir);
	return (gchar *)-1;
}

void
update_stats(gboolean command)
{
	FILE *STF;
	gchar buf[512];
	int spam=0;
	gchar *stfile;

	//update stats
	if (command)
		stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rspam", NULL);
	else
		stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		goto create;
	fgets(buf, 512, STF);
	fclose(STF);
	spam = atoi(buf);
create:	spam++;
	STF = fopen(stfile, "w+");
	if (!STF)
		return;
	fprintf(STF, "%d", spam);
	fclose(STF);
}

gint
rspam_command(CamelMimeMessage *msg, gchar *name, gint command) //command: 1 - spam; 0 - ham
{
	gint i;
	gchar *cmd, *cmdp;
	int fds[2];
	pid_t pid;

	//later we have to check if we have a home and we're registered
	//remember to put -s when testing, in order no to clog cloudmark
	for (i=0; i < CNETS; i++) {
		g_print("CNET: %s\n", cnets[i][0]);
		if (gconf_client_get_bool (rspam_gconf, cnets[i][3], NULL)) {
			g_print("%s is enabled.\n", cnets[i][0]);
			if (command) {	/*spam*/
				if (!strncmp(cnets[i][0], "spamcop", 7))
					intern_call((void *)cnets[i][1], msg, name);
				else {
					if (rspam_online) {
						if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
							cmdp = g_strconcat(cnets[i][1], " < %s &", NULL);
							cmd = g_strdup_printf(cmdp, name);
							system(cmd);
							g_free(cmd);
							g_free(cmdp);
							taskbar_push_message(_("Message reported as SPAM!"));
							update_stats(command);
						} else
						taskbar_push_message(_("Message reported as SPAM! (dry mode!)"));
					} else
						taskbar_push_message(_("Offline mode! Not reporting SPAM!"));

				}
			} else {		/*ham*/
				if (rspam_online) {
					if (NULL != cnets[i][2]) {
						if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
							cmdp = g_strconcat(cnets[i][2], " < %s", NULL);
							cmd = g_strdup_printf(cmdp, name);
							system(cmd);
							g_free(cmd);
							g_free(cmdp);
						}
						taskbar_push_message(_("Message is no loger SPAM!"));
					} else {
						gchar *msg =
						g_strdup_printf(
							_("Revoke operation not supported on %s!"),
							cnets[i][0]);
						taskbar_push_message(msg);
						g_free(msg);
#ifdef DEBUG
						g_print("revoke not supported!\n");
#endif
					}
					update_stats(command);
				}
				else
					taskbar_push_message(_("Offline mode. Cannot revoke SPAM."));
			}
		} else
			g_print("%s is disabled. Skip test.\n", cnets[i][0]);
	}
end:	//let's just make sure we don't leave any junk on our way out,
	//even if we're in trash pile
	tlist = g_list_append(tlist, g_strdup(name));
	return 1;
}


void
spamcop_report(CamelMimeMessage *msg, gchar *msgfile)
{
	CamelStream *mem;
	GByteArray *byte_array;
	int tzone;
	time_t date, now;
	EAccount *account = NULL;
	gchar *from, *descr;
	CamelFolder *mail_folder;
	CamelMessageInfo *info;
	struct _send_data *send;
	struct _camel_header_raw *xev;
	GtkWidget *composer;
	CamelMimePart *part;
	CamelMimeMessage *new = camel_mime_message_new();
	gchar *text, *tmsg;
	CamelMultipart *body = camel_multipart_new();
	CamelInternetAddress *addr;
	gchar *sc_email;
	gssize len;

	date = camel_mime_message_get_date_received(msg, &tzone);

	/*Convert to UTC*/
	date += (tzone / 100) * 60 * 60;
	date += (tzone % 100) * 60;
	time(&now);

	if (date && date < now - 2*86400) {
		taskbar_push_message(
			_("SpamCop: Message too old, not reporting mail older than 2 days."));
#ifdef DEBUG
		g_print("SpamCop: Message too old, not reporting mail older than 2 days.\n");
#endif
		d(g_print("now:%d\n", (guint)now - 2*86400));
		d(g_print("date:%d\n", (guint)date));
		goto fail;
	}

#if EVOLUTION_VERSION < 22900
	if (!em_utils_check_user_can_send_mail(NULL)) {
		e_error_run(NULL,
			"mail:send-no-account-enabled",
			NULL);
#else
	if (!em_utils_check_user_can_send_mail()) {
		e_alert_run_dialog_for_args(
			e_shell_get_active_window (NULL),
			"mail:send-no-account-enabled",
			NULL);
#endif
		goto fail;
	}

#if EVOLUTION_VERSION < 22900
	account = mail_config_get_default_account ();
#else
	account = e_get_default_account ();
#endif

	if ( !account || (account->transport == NULL)
		|| (account->transport->url == NULL)
		|| (*account->transport->url == '\0'))
		goto fail;

	// check and trim if message is above spamcop limit 50k
	// couldn't find better method at this time :-)
#if EVOLUTION_VERSION < 22304
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL);
#else
#if EVOLUTION_VERSION < 22900
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL, NULL);
#else
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL, NULL, NULL);
#endif
#endif
	if ( SPAMCOP_LIMIT < len) {
		tmsg = g_strndup(text,
			SPAMCOP_LIMIT);
		gchar *tmp = g_strconcat(
				(gchar *)tmsg,
				"[message truncated by evolution RSPAM]",
				NULL);
		g_free(tmsg);
		mem = camel_stream_mem_new();
#if (DATASERVER_VERSION >= 2033001)
		camel_stream_write (mem, tmp, strlen(tmp), NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
		camel_stream_write(mem, tmp, strlen(tmp));
		camel_stream_reset(mem);
#else
		camel_stream_write(mem, tmp, strlen(tmp), NULL);
		camel_stream_reset(mem, NULL);
#endif
#endif
		g_free(tmp);
		msg = camel_mime_message_new();
#if (DATASERVER_VERSION >= 2033001)
		camel_data_wrapper_construct_from_stream_sync (
			CAMEL_DATA_WRAPPER(msg), mem, NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
        camel_data_wrapper_construct_from_stream (
			CAMEL_DATA_WRAPPER(msg), mem);
#else
        camel_data_wrapper_construct_from_stream (
			CAMEL_DATA_WRAPPER(msg), mem, NULL);
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(mem);
#else
		camel_object_unref(mem);
#endif
	}
	g_free(text);

#if EVOLUTION_VERSION < 22900
		mail_folder = mail_component_get_folder(
				NULL,
				MAIL_COMPONENT_FOLDER_OUTBOX);
#else
#if EVOLUTION_VERSION < 29102
		mail_folder = e_mail_local_get_folder(E_MAIL_FOLDER_OUTBOX);
#else
		mail_folder = e_mail_local_get_folder(E_MAIL_LOCAL_FOLDER_OUTBOX);
#endif

#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_ref(mail_folder);
#else
		camel_object_ref(mail_folder);
#endif

		info = camel_message_info_new(NULL);
		camel_message_info_set_flags(
			info,
			CAMEL_MESSAGE_SEEN,
			~0);

		camel_mime_message_set_subject(
			new,
			"report spam");

		addr = camel_internet_address_new();
		sc_email = gconf_client_get_string(
				rspam_gconf,
				GCONF_KEY_SPAMCOP_EMAIL,
				NULL);
		camel_address_decode(
			CAMEL_ADDRESS(addr),
			sc_email ? sc_email : SPAMCOP_DEFAULT_EMAIL);
		camel_mime_message_set_recipients(
			new,
			CAMEL_RECIPIENT_TYPE_TO,
			addr);
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(addr);
#else
		camel_object_unref(addr);
#endif

		addr = camel_internet_address_new();
		d(g_print("name:%s\n", account->id->name));
		d(g_print("address:%s\n", account->id->address));
		camel_internet_address_add(
			addr,
			account->id->name,
			account->id->address);
		camel_mime_message_set_from(
			new,
			addr);
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(addr);
#else
		camel_object_unref(addr);
#endif

		xev = mail_tool_remove_xevolution_headers (msg);
		camel_header_raw_clear(&xev);

		camel_medium_remove_header (
			CAMEL_MEDIUM (msg),
			"Bcc");

	part = camel_mime_part_new ();
#if (DATASERVER_VERSION >= 2031001)
	camel_medium_set_content (
		CAMEL_MEDIUM (part),
		CAMEL_DATA_WRAPPER (msg));
#else
	camel_medium_set_content_object (
		CAMEL_MEDIUM (part),
		CAMEL_DATA_WRAPPER (msg));
#endif

	camel_mime_part_set_content_type(
		part,
		"message/rfc822;"
		" x-spam-type=report");
	descr = g_strdup_printf(
		"spam report via RSPAM-%s",
		VERSION);
	camel_mime_part_set_description(
		part,
		descr);
	camel_mime_part_set_disposition(
		part,
		"attachment");
	camel_medium_set_header(
		CAMEL_MEDIUM(part),
		"Content-Transfer-Encoding",
		"8bit");
//    camel_medium_set_header(part, "X-Spam-Relays-Trusted", "");
//    camel_medium_set_header(part, "X-Spam-Relays-Untrusted", "");

	camel_data_wrapper_set_mime_type(
		CAMEL_DATA_WRAPPER(body),
		"multipart/mixed;"
		"boundary=\"\"");
	camel_multipart_set_boundary(
		body,
		NULL);
	camel_multipart_add_part(
		body,
		part);
#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(part);
#else
	camel_object_unref(part);
#endif

	/*more voodoo here*/

	if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
#if (DATASERVER_VERSION >= 2031001)
		camel_medium_set_content(
			CAMEL_MEDIUM(new),
			CAMEL_DATA_WRAPPER(body));
#else
		camel_medium_set_content_object(
			CAMEL_MEDIUM(new),
			CAMEL_DATA_WRAPPER(body));
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(body);
#else
		camel_object_unref(body);
#endif

#if EVOLUTION_VERSION < 29102
		mail_append_mail(mail_folder,
			new, info, NULL, NULL);
#else
		e_mail_folder_append_message(
			mail_folder, new, info,
			G_PRIORITY_DEFAULT, NULL,
			NULL, NULL);
#endif
		/*composer_send_queued_cb*/

		update_stats(1);
	}
fail:	return;
}

guint
read_spam(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rspam", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

guint
read_ham(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

void rspam_action_do (CamelFolder *folder, const char *uid, CamelMimeMessage *msg, void *data)
{
	gchar *name;
	gint i;

	name = save_message(msg);
	if (name) {
		rspam_command(msg, name, 1);
		g_free(name);
	}

	if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL)) {
//		camel_folder_freeze(folder);
		d(g_print("uid:%s\n", uid));
		camel_folder_set_message_flags(folder, uid,
			CAMEL_MESSAGE_JUNK,
			CAMEL_MESSAGE_JUNK);
//		camel_folder_sync_message(folder, uid, NULL);
//		mail_refresh_folder(folder, NULL, NULL);
//		camel_folder_sync (folder, TRUE, NULL);
//		camel_folder_thaw(folder);
//		camel_operation_end(NULL);
	}
}

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t);
#else
org_gnome_sa_rspam (GtkAction *action,
		EShellView *shell_view);
#endif

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t)
#else
org_gnome_sa_rspam(GtkAction *action, EShellView *shell_view)
#endif
{
	CamelFolder *folder;
	GPtrArray *uids;
	gint i;
#if EVOLUTION_VERSION < 22900
	uids = t->uids;
	folder = t->folder;
#else
	EShellContent *shell_content;
	EMailReader *reader;

	shell_content = e_shell_view_get_shell_content (shell_view);

	reader = E_MAIL_READER (shell_content);
	folder = e_mail_reader_get_folder(reader);
	uids = e_mail_reader_get_selected_uids(reader);
#endif

	while (gtk_events_pending())
		gtk_main_iteration ();

	for (i = 0; i < (uids ? uids->len : 0); i++) {
		d(g_print("i:%d, uids:%s\n", i, uids->pdata[i]));
		mail_get_message (
			folder,
			uids->pdata[i],
			rspam_action_do, NULL, mail_msg_unordered_push);
	}

	while (gtk_events_pending())
		gtk_main_iteration ();
}

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t);
#else
org_gnome_sa_revoke (GtkAction *action,
		EShellView *shell_view);
#endif

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t)
#else
org_gnome_sa_revoke (GtkAction *action,
		EShellView *shell_view)
#endif
{
	CamelFolder *folder;
	GPtrArray *uids;
	CamelMimeMessage *msg;
	gchar *name;
	gint i;
#if EVOLUTION_VERSION < 22900
	uids = t->uids;
	folder = t->folder;
#else
	EShellContent *shell_content;
	EMailReader *reader;
	GtkWidget *message_list;
	shell_content = e_shell_view_get_shell_content (shell_view);

	reader = E_MAIL_READER (shell_content);
	message_list = e_mail_reader_get_message_list (reader);
	folder = MESSAGE_LIST(message_list)->folder;
	uids = message_list_get_selected (MESSAGE_LIST(message_list));
#endif

	while (gtk_events_pending())
		gtk_main_iteration ();

	for (i = 0; i < (uids ? uids->len : 0); i++) {
#if (DATASERVER_VERSION >= 2033001)
		msg = camel_folder_get_message_sync (folder,
			g_ptr_array_index(uids, i), NULL, NULL);
#else
		msg = camel_folder_get_message (folder,
			g_ptr_array_index(uids, i), NULL);
#endif
		if (!msg)
			continue;

	name = save_message(msg);
	if (name) {
		rspam_command(msg, name, 0);
		g_free(name);
	}

	if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL))
		camel_folder_set_message_flags(
			folder, g_ptr_array_index(uids, i),
			CAMEL_MESSAGE_SEEN|CAMEL_MESSAGE_JUNK,
			CAMEL_MESSAGE_SEEN);

#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(msg);
#else
	camel_object_unref(msg);
#endif
	}

	while (gtk_events_pending())
		gtk_main_iteration ();
}


gchar*
pyzor_discover_cb (GtkWidget *widget, gpointer data)
{
	GByteArray *output_buffer = g_byte_array_new ();
	int result, status, errnosav, fds[2], out_fds[2];
	CamelStream *stream;
	CamelStream *stream2;
	char *program;
	pid_t pid;
	int i=0;
	char *argv[30];

	if (!rspam_online)
		return 0;

	argv[i++] = "pyzor";
	argv[i++] = "discover";
	argv[i] = NULL;

	if (output_buffer && pipe (out_fds) == -1) {
		return 0;
	}

	if (!(pid = fork ())) {
		/* child process */
		int maxfd, fd;

		if (dup2 (out_fds[1], STDOUT_FILENO) == -1)
			_exit (1 & 0377);
		if (output_buffer)
			close (out_fds [1]);

		setsid ();

		maxfd = sysconf (_SC_OPEN_MAX);
		for (fd = 3; fd < maxfd; fd++)
			fcntl (fd, F_SETFD, FD_CLOEXEC);

		execvp (argv[0], argv);
		_exit (1 & 0377);
	} else if (pid < 0) {
		return 0;
	}

	if (output_buffer)
		close (out_fds [1]);

	if (output_buffer) {
		CamelStreamMem *memstream;

		stream = camel_stream_fs_new_with_fd (out_fds[0]);

		memstream = (CamelStreamMem *) camel_stream_mem_new ();
		camel_stream_mem_set_byte_array (memstream, output_buffer);

#if (DATASERVER_VERSION >= 2033001)
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream, NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream);
#else
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream, NULL);
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref (stream);
#else
		camel_object_unref (stream);
#endif
		g_byte_array_append (output_buffer, "", 1);

		printf (
			"child process output: %s len: %d\n",
			output_buffer->data, output_buffer->len);
	}
	gchar *tmp = g_strdup(output_buffer->data);

	return tmp;
}

static void
discover_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gchar *cmd;
	if (!rspam_online)
		return;

	const gchar *home = stuff->homepath;
	const gchar *user = stuff->user;
	const gchar *pass = stuff->pass;
	if (home && user && pass)
		cmd = g_strdup_printf(
			"razor-admin -home %s -discover -user %s -pass %s &",
			home, user, pass);
	else
		cmd = g_strdup("razor-admin -discover &");
#ifdef DEBUG
	g_print("Razor discover - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
}

static void
register_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gchar *cmd;
	if (!rspam_online)
		return;

	const gchar *home = gtk_entry_get_text(GTK_ENTRY(stuff->home));
	const gchar *user = gtk_entry_get_text(GTK_ENTRY(stuff->username));
	const gchar *pass = gtk_entry_get_text(GTK_ENTRY(stuff->password));
	cmd = g_strdup_printf(
		"razor-admin -d -home %s -register -user %s -pass %s",
		home, user, pass);
#ifdef DEBUG
	g_print("Razor register - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
	readrazorconfig(stuff);
}

static void
razor_home_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *razor_home = gtk_entry_get_text(GTK_ENTRY(widget));

	/* Save the new setting to gconf */
	gconf_client_set_string (
		stuff->target->gconf,
		GCONF_KEY_RAZOR_HOME, razor_home, NULL);
}

static void
spamcop_email_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *spamcop_email = gtk_entry_get_text(GTK_ENTRY(widget));

	/* Save the new setting to gconf */
	gconf_client_set_string (
		stuff->target->gconf,
		GCONF_KEY_SPAMCOP_EMAIL, spamcop_email, NULL);
}

static void
enable_dcc_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_ENABLE_DCC, active, NULL);
}

static void
enable_razor_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf,
		GCONF_KEY_ENABLE_RAZOR, active, NULL);
}

static void
enable_spamcop_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf,
		GCONF_KEY_ENABLE_SPAMCOP, active, NULL);
}

static void
enable_movejunk_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_MOVE_JUNK, active, NULL);
}

static void
enable_dryrun_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_DRY_RUN, active, NULL);
}

static void
enable_pyzor_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting to gconf */
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_ENABLE_PYZOR, active, NULL);
}

static void
readrazorconfig(gpointer data)
{
	gchar buf[512];
	char **str2;
	FILE *file;
	gchar *key, *path;
	gchar *str, *cmd;

	struct rspam_stuff *stuff = (struct rspam_stuff *) data;

	//we might had razor before, so try to fetch it's home
	//otherwise we'll build ovious razor home from homedir
	key = gconf_client_get_string(
		rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
	if (!key) {
		str = g_build_filename(
			g_get_home_dir(), ".razor", "identity", NULL);
		gchar *tmp = g_path_get_dirname(str);
		gconf_client_set_string (
			rspam_gconf, GCONF_KEY_RAZOR_HOME, tmp, NULL);
		g_free(tmp);
		key = gconf_client_get_string(
			rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
	} else {
		//we  have a razor home stored
		str = g_build_filename(key, "identity", NULL);
	}
	stuff->homepath = key;

	//try to see if files really exists
	file = fopen(str, "rb");
	if (!file) {
		//identity file does not exist, let's try home
		path = g_path_get_dirname(str);
		if (!g_file_test(path, G_FILE_TEST_IS_DIR)) {
			//create
			cmd = g_strdup_printf(
				"razor-admin -d -home %s -create",
				path);
		g_print("%s\n", cmd);
		system(cmd);
		g_free(cmd);
		}
		//register
		cmd = g_strdup_printf("razor-admin -d -home %s -register",
				path);
		g_print("%s\n", cmd);
		system(cmd);
		g_free(cmd);
		//file must now be created
		//to we should check status
		file = fopen(str, "rb");
		if (!file) {
			stuff->user = NULL;
			stuff->pass = NULL;
			return;
		}
	}

	while(fgets(buf, 512, file)) {
		g_strstrip(buf);
		str2 = g_strsplit(buf, "=", 0);
		g_strstrip(str2[1]);
		if (!strncmp(str2[0], "user", 4))
			stuff->user = str2[1];
		if (!strncmp(str2[0], "pass", 4))
			stuff->pass = str2[1];
	}
	fclose(file);
}

void
taskbar_push_message(gchar *message)
{
#if 0
	EActivityHandler *activity_handler =
			mail_component_peek_activity_handler (
				mail_component_peek ());
	e_activity_handler_set_message(activity_handler, message);
#endif
}

void
taskbar_pop_message(void)
{
#if EVOLUTION_VERSION < 22900 //kb//
	EActivityHandler *activity_handler =
			mail_component_peek_activity_handler (
						mail_component_peek ());
	e_activity_handler_unset_message(activity_handler);
#else
	EShellTaskbar *shell_taskbar;
	g_return_if_fail(main_shell_view != NULL);
	shell_taskbar = e_shell_view_get_shell_taskbar (main_shell_view);
	e_shell_taskbar_set_message (shell_taskbar, "");
#endif

}

//=================
//== guification ==
//=================

GtkWidget *
rspam_page_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_data)
{
	struct rspam_stuff *stuff;
	EMConfigTargetPrefs *target = (EMConfigTargetPrefs *) hook_data->config->target;
	GtkWidget *page;
	GtkWidget *tab_label;
	GtkWidget *frame;
	GtkWidget *frame_label;
	GtkWidget *padding_label;
	GtkWidget *hbox;
	GtkWidget *inner_vbox;
	GtkWidget *check;
	GtkWidget *option;
	GtkWidget *check_gaim;
	GtkWidget *button;
	GtkWidget *button2;
	GtkWidget *entry;
	GtkWidget *entry1,*entry2,*entry3;
	GtkWidget *label1, *label2, *label3;
	GtkWidget *table1;
	gchar     *str;
	GtkTooltips *tips;

	tips = gtk_tooltips_new ();

	/* A structure to pass some stuff around */
	stuff = g_new0 (struct rspam_stuff, 100);
	stuff->target = target;

	readrazorconfig(stuff);

	/* Create a new notebook page */
	page = gtk_vbox_new (FALSE, 0);
	GTK_CONTAINER (page)->border_width = 12;
	tab_label = gtk_label_new (_("Report Spam"));
	gtk_notebook_append_page (
		GTK_NOTEBOOK (hook_data->parent), page, tab_label);

	/* Frame */
	frame = gtk_vbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);

	/* "Filtering Network" */
	frame_label = gtk_label_new ("");
	str = g_strdup_printf (
		"<span weight=\"bold\">%s</span>", _("Filtering Network"));
	gtk_label_set_markup (GTK_LABEL (frame_label), str);
	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
	gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	/* Indent/padding */
	hbox = gtk_hbox_new (FALSE, 12);
	gtk_box_pack_start (GTK_BOX (frame), hbox, FALSE, TRUE, 0);
	padding_label = gtk_label_new ("");
	gtk_box_pack_start (GTK_BOX (hbox), padding_label, FALSE, FALSE, 0);
	inner_vbox = gtk_vbox_new (FALSE, 6);
	gtk_box_pack_start (GTK_BOX (hbox), inner_vbox, FALSE, FALSE, 0);

	check = gtk_check_button_new_with_mnemonic (
			_("_Vipul's Razor (Razor)"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_RAZOR, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_razor_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Vipul's Razor is a distributed, collaborative, spam detection and filtering network. Through user contribution, Razor establishes a distributed and constantly updating catalogue of spam in propagation."), NULL);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	check = gtk_check_button_new_with_mnemonic (_("_Pyzor"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_PYZOR, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_pyzor_cb), stuff);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Python implementation of Razor network."), NULL);
	check = gtk_check_button_new_with_mnemonic (
			_("_Distributed Checksum Clearinghouse (DCC)"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_DCC, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_dcc_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("A DCC server totals reports of checksums of messages from clients and answers queries about the total counts for checksums of mail messages."), NULL);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);
	check = gtk_check_button_new_with_mnemonic (_("_SpamCop.Net"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_SPAMCOP, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_spamcop_cb), stuff);
	gtk_box_pack_start (GTK_BOX (inner_vbox), check, FALSE, FALSE, 0);


	check = gtk_check_button_new_with_mnemonic (_("Move to Junk folder after report"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_MOVE_JUNK, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_movejunk_cb), stuff);

	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Moves message to Junk Folder after spam report without affecting spam learing (see SA plugin)"), NULL);
	gtk_box_pack_start (GTK_BOX (frame), check, FALSE, FALSE, 0);

	check = gtk_check_button_new_with_mnemonic (_("Dry-run"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		gconf_client_get_bool (target->gconf, GCONF_KEY_DRY_RUN, NULL));
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_dryrun_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), check,
		_("Go to all phases of spam reporting but do not actualy connect to reporting servers."), NULL);
	gtk_box_pack_start (GTK_BOX (frame), check, FALSE, FALSE, 0);

	/* "Razor Settings" */
	frame_label = gtk_label_new ("");
	str = g_strdup_printf (
		"<span weight=\"bold\">%s</span>", _("Razor Settings"));
	gtk_label_set_markup (GTK_LABEL (frame_label), str);
	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
	gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	table1 = gtk_table_new (4, 2, FALSE);
	gtk_container_add (GTK_CONTAINER (frame), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

	label1 = gtk_label_new_with_mnemonic (_("Razor _Home: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

	label2 = gtk_label_new (_("Username: "));
	gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

	label3 = gtk_label_new (_("Password: "));
	gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 2, 3,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

	entry1 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
	gtk_entry_set_text(
		GTK_ENTRY(entry1),
		gconf_client_get_string(target->gconf, GCONF_KEY_RAZOR_HOME, NULL));
	g_signal_connect (
		GTK_ENTRY (entry1), "changed",
		G_CALLBACK (razor_home_cb), stuff);
	stuff->home = entry1;

	entry2 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_text(GTK_ENTRY(entry2), (gpointer)stuff->user);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry2), 8226);
	stuff->username = entry2;

	entry3 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 2, 3,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_visibility (GTK_ENTRY (entry3), FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry3), (gpointer)stuff->pass);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry3), 8226);
	stuff->password = entry3;

	button = gtk_button_new_with_mnemonic(_("_Register"));
	gtk_table_attach (GTK_TABLE (table1), button, 0, 1, 3, 4,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	g_signal_connect (
		GTK_BUTTON (button),
		"clicked", G_CALLBACK (register_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), button,
		_("Registers a new identity, used for authenticating with Razor Nomination Servers. razor-admin is required in $PATH."), NULL);

	button2 = gtk_button_new_with_mnemonic(_("_Discover"));
	gtk_table_attach (GTK_TABLE (table1), button2, 0, 1, 4, 5,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	g_signal_connect (
		GTK_BUTTON (button2), "clicked",
		G_CALLBACK (discover_cb), stuff);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), button2,
		_("Force discovery. This will create server.*.lst files in Razor Home. razor-admin is required in $PATH"), NULL);

	/* "SpamCop Settings" */
	frame_label = gtk_label_new ("");
	str = g_strdup_printf (
		"<span weight=\"bold\">%s</span>", _("SpamCop Settings"));
	gtk_label_set_markup (GTK_LABEL (frame_label), str);
	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
	gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	table1 = gtk_table_new (4, 2, FALSE);
	gtk_container_add (GTK_CONTAINER (frame), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

	label1 = gtk_label_new_with_mnemonic (_("Report Address: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

	entry1 = gtk_entry_new ();
	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
	gchar *sc_email = gconf_client_get_string(
				target->gconf,
				GCONF_KEY_SPAMCOP_EMAIL,
				NULL);
	gtk_entry_set_text(GTK_ENTRY(entry1),
		(sc_email ? sc_email : SPAMCOP_DEFAULT_EMAIL));
	g_signal_connect (
		GTK_ENTRY (entry1),
		"changed", G_CALLBACK (spamcop_email_cb), stuff);
	stuff->spamcop_email = sc_email ? sc_email : "rspam-submit@spam.spamcop.net";
	gtk_tooltips_set_tip (GTK_TOOLTIPS (tips), entry1,
		_("By default plugin uses a default email address. However you are encouraged to register to http://spamcop.net because your report will weight more using your account."), NULL);

/*	frame_label = gtk_label_new ("");
        str = g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Pyzor Statistics"));
        gtk_label_set_markup (GTK_LABEL (frame_label), str);
        g_free (str);
        GTK_MISC (frame_label)->xalign = 0.0;
        gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

        table1 = gtk_table_new (1, 2, FALSE);
        gtk_container_add (GTK_CONTAINER (frame), table1);
        gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

        label1 = gtk_label_new_with_mnemonic (_("Server: "));
        gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        label1 = gtk_label_new_with_mnemonic (pyzor_status);
        gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
        gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

        gtk_box_pack_start (GTK_BOX (frame), table1, FALSE, FALSE, 0);*/

	frame_label = gtk_label_new ("");
	str = g_strdup_printf (
		"<span weight=\"bold\">%s</span>",
		_("Spam Statistics"));
	gtk_label_set_markup (GTK_LABEL (frame_label), str);
	g_free (str);
	GTK_MISC (frame_label)->xalign = 0.0;
	gtk_box_pack_start (GTK_BOX (frame), frame_label, FALSE, FALSE, 0);

	table1 = gtk_table_new (2, 2, FALSE);
	gtk_container_add (GTK_CONTAINER (frame), table1);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 4);

	label1 = gtk_label_new_with_mnemonic (_("Spam reported: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

	label1 = gtk_label_new_with_mnemonic (_("Spam revoked: "));
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

	guint spam = read_spam();
	gchar *tmp = g_strdup_printf(_("%d messages"), spam?spam:0);
	label1 = gtk_label_new_with_mnemonic (tmp);
	gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 0, 1,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	g_free(tmp);

	spam = read_ham();
	tmp = g_strdup_printf(_("%d messages"), spam?spam:0);
	label1 = gtk_label_new_with_mnemonic (tmp);
	gtk_table_attach (GTK_TABLE (table1), label1, 1, 2, 1, 2,
		(GtkAttachOptions) (GTK_FILL),
		(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	g_free(tmp);

	/* Clean up */
//	g_signal_connect (page, "destroy", G_CALLBACK (cleanup_cb), stuff);

	gtk_widget_show_all (page);
	return page;
}

#if EVOLUTION_VERSION < 22900
void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t);
#else
void org_gnome_sa_rspam_startup(void *ep, ESEventTargetUpgrade *t);
#endif

#if EVOLUTION_VERSION < 22900
void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t)
#else
void org_gnome_sa_rspam_startup(void *ep, ESEventTargetUpgrade *t)
#endif
{
}

gboolean
check_discovery(void)
{
	gchar *stfile = g_build_filename(
				(gchar *)e_get_user_data_dir(),
				".rspam.disc", NULL);
	struct timeval start;
	FILE *f = NULL;
	gboolean ret = TRUE;
	unsigned long int remain;
	gchar rfeed[80];
	memset(rfeed, 0, 79);
	gettimeofday(&start, NULL);
	if (!g_file_test(stfile, G_FILE_TEST_EXISTS)) {
		if ((f = fopen(stfile, "w"))) {
			fprintf(f, "%lu", start.tv_sec);
			fclose(f);
		}
		ret = TRUE;
		goto out;
	}
	if ((f = fopen(stfile, "r+"))) {
		fgets(rfeed, 50, f);
		remain = start.tv_sec - strtoul(
			(const char *)&rfeed, NULL, 10);
		if (DISCOVERY_TTL <= remain) {
			(void)fseek(f, 0L, SEEK_SET);
			fprintf(f, "%lu", start.tv_sec);
			fclose(f);
			ret =  TRUE;
			goto out;
		} else {
			fclose(f);
			ret = FALSE;
		}
	}
out:    g_free(stfile);
	return ret;
}

#if EVOLUTION_VERSION >= 29101
EMailSession*
_get_mail_session(void);

EMailSession*
_get_mail_session(void)
{
       EMailBackend *backend;
       EMailReader *reader;
       EShellContent *shell_content;
       shell_content = e_shell_view_get_shell_content (main_shell_view);
       reader = E_MAIL_READER (shell_content);
       backend = e_mail_reader_get_backend (reader);
       return e_mail_backend_get_session (backend);
}
#endif

void
#if (DATASERVER_VERSION >= 2031002)
rs_online(EShell *shell)
#else
rs_online(CamelObject *o, void *event_data, void *data)
#endif
{
#if (DATASERVER_VERSION >= 2031002)
	rspam_online =  e_shell_get_online (shell);
#else
	rspam_online =  camel_session_is_online ((CamelSession *)o);
#endif
	if (rspam_online && check_discovery()) {
		while (gtk_events_pending())
			gtk_main_iteration ();
		taskbar_push_message(_("Rspam: Performing discovery (..)"));
		/*discover pyzor*/
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_PYZOR, NULL))
			pyzor_status = pyzor_discover_cb(NULL, NULL);
		/*discover razor*/
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_RAZOR, NULL))
			discover_cb(NULL, stuff);
		taskbar_pop_message();
	while (gtk_events_pending())
		gtk_main_iteration ();
	}
}

static GtkActionEntry entries[] = {
	{ "report-as-spam",
	"stock_spam",
	N_("_Report as SPAM"),
	NULL,
	N_("Report current message to various spam filtering networks"),
	G_CALLBACK (org_gnome_sa_rspam) },

	{ "revoke-spam",
	"mail-mark-notjunk",
	N_("Revo_ke SPAM"),
	NULL,
	N_("Whithdraws reported message as being spam"),
	G_CALLBACK (org_gnome_sa_revoke) },
};

#if EVOLUTION_VERSION >= 22900 //kb//
static EPopupActionEntry popup_entries[] = {
	{ "report-as-spam-popup",
	NULL,
	"report-as-spam" },

	{ "revoke-spam-popup",
	NULL,
	"revoke-spam" },
};

gboolean
e_plugin_ui_init (GtkUIManager *ui_manager,
		EShellView *shell_view);

gboolean
e_plugin_ui_init (GtkUIManager *ui_manager,
		EShellView *shell_view)
{
	EShellWindow *shell_window;
	GtkActionGroup *action_group;

	shell_window = e_shell_view_get_shell_window (shell_view);
	action_group = e_shell_window_get_action_group (
				shell_window,
				"mail");
	main_shell_view = shell_view;

#if (DATASERVER_VERSION >= 2031002)
	g_signal_connect(
#if EVOLUTION_VERSION < 22900
		mail_component_peek_session(NULL),
#else
		e_shell_get_default(),
#endif
		"notify::online",
		G_CALLBACK(rs_online),
		NULL);
	/*shell doesn't trigger online event ?*/
	rs_online(e_shell_get_default());
#else
	camel_object_hook_event(
#if EVOLUTION_VERSION < 22900
		mail_component_peek_session(NULL),
#else
		session,
#endif
		"online",
		(CamelObjectEventHookFunc)rs_online,
		NULL);
#endif

	gtk_action_group_add_actions (
		action_group,
		entries,
		G_N_ELEMENTS (entries),
		shell_view);
	e_action_group_add_popup_actions (
		action_group,
		popup_entries,
		G_N_ELEMENTS (popup_entries));

//        g_signal_connect (
//              shell_view, "update-actions",
//            G_CALLBACK (update_actions_cb), NULL);

	return TRUE;
}
#endif

void
rspam_finalize(void)
{
	g_print("RSPAM: cleaning temp files.\n");
	g_list_foreach(tlist, (GFunc)unlink, NULL);
	g_list_foreach(tlist, (GFunc)g_free, NULL);
}

#if EVOLUTION_VERSION < 22900
int e_plugin_lib_enable(EPluginLib *ep, int enable);
#else
int e_plugin_lib_enable(EPlugin *ep, int enable);
#endif

int
#if EVOLUTION_VERSION < 22900
e_plugin_lib_enable(EPluginLib *ep, int enable)
#else
e_plugin_lib_enable(EPlugin *ep, int enable)
#endif
{
	if (enable) {
		bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
		bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
		printf("Rspam Plugin enabled (evolution %s, evolution-rspam %s)\n",
			EVOLUTION_VERSION_STRING,
			VERSION);
		rspam_gconf = gconf_client_get_default();
		/* A structure to pass some stuff around */
		stuff = g_new0 (struct rspam_stuff, 100);

		readrazorconfig(stuff);
		atexit(rspam_finalize);
	} else {
		printf("Rspam Plugin disabled\n");
	}

	return 0;
}
